/*
 * (c) Copyright 2004 Frank Yeah (pwipwi@ms66.hinet.net)
 *
 * To anyone who acknowledges that this file is provided "AS IS"
 * without any express or implied warranty:
 *     permission to use, copy, modify, and distribute this file
 * for any purpose is hereby granted without fee, provided that
 * the above copyright notice and this notice appears in all
 * copies. Frank Yeah makes no representations about the suitability
 * of this software for any purpose.
 */

//---------------------------------------------------------------------------

#ifndef CalculatorH
#define CalculatorH
//---------------------------------------------------------------------------
#include <list>
#include <iostream>
#include <sstream>
#include "Polynomial.h"
using namespace std;
//---------------------------------------------------------------------------
class Token{
        friend ostream &operator<<(ostream &out,const Token &ta);
    private:
        char c_operator;
        string s_number;
    public:
        Token():c_operator(0){;}
        Token(const string &s_data)
        {
        if(isdigit(s_data[0]) || s_data[0] == 'x' || s_data[0] == '.')
            setNumber(s_data);
        else
            setOperator(s_data[0]);
        }
        void setOperator(const char ca){c_operator = ca;}
        void setNumber(const string &sa){s_number = sa;c_operator = 0;}
        bool isOperator(void) const {return c_operator != 0;}
        const string &getNumber(void) const {return s_number;}
        const char getOperator(void) const {return c_operator;}
        unsigned char getICP()const{return getICP(c_operator);}
        unsigned char getISP()const{return getISP(c_operator);}
        unsigned char getICP(char) const;
        unsigned char getISP(char) const;
        bool operator==(const char c_ioperator)const{return c_operator == c_ioperator;}
        bool operator!=(const char c_ioperator)const{return c_operator != c_ioperator;}
            };


template<class T>
class calculator;
template<class T>
istream &operator>>(istream &in,calculator<T> &ca);
template<class T>
class calculator{

        friend ostream &operator<< <>(ostream &out,const calculator<T> &ca);
        friend istream &operator>> <>(istream &in,calculator<T> &ca);
    private:
        list<Token> l_postfix;
        const char *getToken(Token &t_token,const char *c_begin);
        bool isNumber(char ca) const{return (::isdigit(ca) || ca == 'x' || ca == '.');}
        void processExpression(string &s_expression);
        void translateToPostfix(string s_expression);
    public:
        T Result(void);
                };

//---------------------------------------------------------------------------
template<class T>
const char *calculator<T>::getToken(Token &t_token,const char *c_begin)
{
static unsigned char c_lastisp = 1;
const char *ca = c_begin,*cb = 0;
if(!isdigit(*c_begin) && *c_begin != 'x' && *c_begin != '.')
    {
    t_token.setOperator(*c_begin);
    c_lastisp = t_token.getISP();
    return c_begin+1;
    }
while(*ca != 0)
    {
    if(*ca == 'x')
        {
        if(*(ca+1) == '^' && isdigit(*(ca+2)))
            ca++;
        ca++;
        }
    if(!isdigit(*ca) && *ca != '.')
        {
        if(c_lastisp >=3)
            break;
        else
            {
            if(t_token.getISP(*ca)==2)
                cb = ca;
            else
                {
                if(cb && (*ca) != ')')
                    ca = cb;
                break;
                }
            }
        }
    ca++;
    }
string sa(c_begin,ca);
t_token.setNumber(sa);
if(*ca == 0)
    c_lastisp = 1;
else
    c_lastisp = t_token.getISP();
return ca;
}
//---------------------------------------------------------------------------
template<class T>
void calculator<T>::processExpression(string &s_expression)
{
int i_count = 0;
char *ca = s_expression.begin();
if(ca == s_expression.end())
    throw runtime_error("Illegal expression");
while(ca != s_expression.end() && *ca != 0)
    {
    if(!isNumber(*ca) && *ca != '+' && *ca != '-' && *ca != '*' && *ca != '/' &&
        *ca != '(' && *ca != ')' && *ca != '^')
        {
        throw runtime_error("Illegal expression");
        }
    if(*ca == '(')
        i_count++;
    if(*ca == ')')
        i_count--;
    if(*ca == '-' &&
        (ca == s_expression.begin() || !isNumber(*(ca-1))))
        {
        s_expression.insert(ca,'0');
        ca = s_expression.begin();
        }
    if(*ca == '(' && ca != s_expression.begin() && (isNumber(*(ca-1)) || *(ca-1) == ')'))
        {
        s_expression.insert(ca,'*');
        ca = s_expression.begin();
        }
    if(*ca == ')' && ca != s_expression.end()-1 && (isNumber(*(ca+1)) || *(ca+1) == '('))
        {
        ca++;
        s_expression.insert(ca,'*');
        ca = s_expression.begin();
        }
    ca++;
    }
if(i_count != 0)
    throw runtime_error("Illegal expression");
}
//---------------------------------------------------------------------------
template<class T>
void calculator<T>::translateToPostfix(string s_expression)
{
Token ta;
vector<Token> v_stack;
const char *ca;
processExpression(s_expression);
ca = &s_expression[0];
l_postfix.resize(0);
do
    {
    ca = calculator<T>::getToken(ta,ca);
    if(ta.isOperator())
        {
        if(ta == ')')
            {
            while(v_stack[v_stack.size()-1] != '(')
                {
                l_postfix.push_back(v_stack[v_stack.size()-1]);
                v_stack.pop_back();
                }
            v_stack.pop_back();
            }
        else
            {
            while(v_stack.size() >= 1 &&
                    v_stack[v_stack.size()-1].getISP() >= ta.getICP())
                {
                l_postfix.push_back(v_stack[v_stack.size()-1]);
                v_stack.pop_back();
                }
            v_stack.push_back(ta);
            }
        }
    else
        l_postfix.push_back(ta);
    }
    while(*ca != 0);
while(v_stack.size() >= 1)
    {
    l_postfix.push_back(v_stack[v_stack.size()-1]);
    v_stack.pop_back();
    }
}
//---------------------------------------------------------------------------
template<class T>
T calculator<T>::Result(void)
{

list<Token>::iterator lia = l_postfix.begin();
list<T>l_temp;
list<T>:: iterator lib;
list<T>:: iterator li1,li2;
T p_temp;
while(lia != l_postfix.end())
    {
    if((*lia).isOperator())
        l_temp.push_back(p_temp);
    else
        l_temp.push_back((*lia).getNumber());
    lia++;
    }
lia = l_postfix.begin();
lib = l_temp.begin();
p_temp = *lib;
do
    {
    //cout << p_temp << endl;
    if((*lia).isOperator() && (*lia) != '@')
        {
        p_temp = (*li1);
        switch((*lia).getOperator())
            {
            case '+':
                p_temp+=(*li2);
                break;
            case '-':
                p_temp-=(*li2);
                break;
            case '*':
                p_temp*=(*li2);
                break;
            case '/':
                p_temp /=(*li2);
                break;
            case '^':
                p_temp ^=(*li2);
                break;
            }
        l_postfix.erase(lia--);
        l_postfix.erase(lia--);
        (*lia).setOperator('@');

        l_temp.erase(lib--);
        l_temp.erase(lib--);
        (*lib) = p_temp;

        lia = l_postfix.begin();
        lib = l_temp.begin();
        }
    else
        {
        li1 = li2;li2 = lib;
        lia++;lib++;
        }
    }
while(lia != l_postfix.end());
return p_temp;
}
//---------------------------------------------------------------------------
template<class T>
istream &operator>>(istream &in,calculator<T>& ca)
{
string temp;
in >> temp;
ca.translateToPostfix(temp);
return in;
}
//---------------------------------------------------------------------------

template<class T>
ostream &operator<<(ostream &out,const Token &ta)
{
out << (ta.c_operator ? ta.c_operator:ta.s_number);
return out;
}

template<class T>
ostream &operator<<(ostream &out,const calculator<T> &ca)
{
list<Token> :: const_iterator lia = ca.l_postfix.begin();
do out << *lia << endl;
while((++lia) != ca.l_postfix.end());
return out;
}
//---------------------------------------------------------------------------
#endif
