/*
 * (c) Copyright 2004 Frank Yeah (pwipwi@ms66.hinet.net)
 *
 * To anyone who acknowledges that this file is provided "AS IS"
 * without any express or implied warranty:
 *     permission to use, copy, modify, and distribute this file
 * for any purpose is hereby granted without fee, provided that
 * the above copyright notice and this notice appears in all
 * copies. Frank Yeah makes no representations about the suitability
 * of this software for any purpose.
 */
//---------------------------------------------------------------------------
#ifndef RationalCPP
#define RationalCPP
//---------------------------------------------------------------------------
#include "rational.h"

template<class T>
rational<T>::rational(const string &sa)
{
string sb = sa;
denominator = 10;
int ia,ib;
if((unsigned int)(ia = sa.find(".")) != string::npos)
    {
    ib = sa.length() - ia - 1;
    if(ib <0)
        ib = 0;
    sb.replace(ia,ib,&sa[ia+1]);
    sb[sb.length()-1] = 0;
    formula::exponent(denominator,ib);
    }
else
    denominator = 1;
numerator = sb.c_str();
Reduce();
}
//---------------------------------------------------------------------------
template<class T>
void rational<T>::Reduce()
{
T temp,m,n;
if(denominator == 1)
    return;
if(numerator == 0)
    {
    denominator = 1;
    return;
    }
if(denominator <0)
    {denominator = -denominator;numerator = -numerator;}
n = numerator;
m = denominator;
n = ((n<0)? (-n):n);
m = ((m<0)? (-m):m);
if(n>m)
	{temp = m;m = n;n = temp;}
if(n == 0)
        {denominator = 1;numerator = 0;return;}
while(n != 0)
        {temp = m;m = n;n = temp;n %= m;}
numerator/=m;
denominator/=m;
}
//---------------------------------------------------------------------------
template<class T>
void rational<T>::operator+=(const rational<T> &ra)
{
if(denominator != ra.denominator)
    {
    T temp = this->denominator;
    temp *= ra.numerator;
    this->numerator *= ra.denominator;
    this->numerator += temp;
    this->denominator *= ra.denominator;
    }
else
    numerator += ra.numerator;
Reduce();
}
//---------------------------------------------------------------------------
template<class T>
void rational<T>::operator-=(const rational<T> &ra)
{
if(denominator != ra.denominator)
    {
    T temp = this->denominator;
    temp *= ra.numerator;
    this->numerator *= ra.denominator;
    this->numerator -= temp;
    this->denominator *= ra.denominator;
    }
else
    numerator -= ra.numerator;
Reduce();
}
//---------------------------------------------------------------------------
template<class T>
void rational<T>::operator*=(const rational<T> &ra)
{
this->numerator *= ra.numerator;
this->denominator *= ra.denominator;
Reduce();
}
//---------------------------------------------------------------------------
template<class T>
void rational<T>::operator/=(rational<T> const &ra)
{
if(ra.numerator == 0)
	throw runtime_error("Divided by zero");
this->numerator *= ra.denominator;
this->denominator *= ra.numerator;
Reduce();
}
//---------------------------------------------------------------------------
template<class T>
bool rational<T>::operator>(const rational<T> &value) const
{
if(this->denominator == value.denominator)
        return this->numerator > value.numerator;
if(this->numerator > 0 && value.numerator < 0)
        return true;
if(this->numerator < 0 && value.numerator > 0)
        return false;
return this->numerator * value.denominator > value.numerator * this->denominator;
}
//---------------------------------------------------------------------------
template<class T>
bool rational<T>::operator<(const rational<T> &value) const
{
if(this->denominator == value.denominator)
        return this->numerator < value.numerator;
if(this->numerator > 0 && value.numerator < 0)
        return false;
if(this->numerator < 0 && value.numerator > 0)
        return true;
return this->numerator * value.denominator < value.numerator * this->denominator;
}                                
#endif
