/*
 * (c) Copyright 2004 Frank Yeah (pwipwi@ms66.hinet.net)
 *
 * To anyone who acknowledges that this file is provided "AS IS"
 * without any express or implied warranty:
 *     permission to use, copy, modify, and distribute this file
 * for any purpose is hereby granted without fee, provided that
 * the above copyright notice and this notice appears in all
 * copies. Frank Yeah makes no representations about the suitability
 * of this software for any purpose.
 */
//---------------------------------------------------------------------------

#ifndef RationalH
#define RationalH
//---------------------------------------------------------------------------
#include <string>
#include "definition.h"

using namespace std;
using namespace def;
template<class T>
class rational{
    private:
        T denominator;
        T numerator;
        void Reduce();
    public:
        rational(int ia = 0):denominator(1),numerator(ia){;}
        rational(long la):denominator(1),numerator(la){;}
        rational(const string& sa);
        rational(const rational<T>& value):
                denominator(value.denominator),numerator(value.numerator){Reduce();}
        rational(const T& idenominator,const T& inumerator):
        	denominator(idenominator),numerator(inumerator){Reduce();}
    public:
        void operator+=(rational<T> const &ra);
        void operator-=(rational<T> const &ra);
        void operator*=(rational<T> const &ra);
        void operator/=(rational<T> const &ra);

        template<typename T2>
        void operator%=(const T2& value){if(denominator != 1)throw runtime_error("Illegal Mod");numerator%=value;}

        rational<T>& operator-(void){numerator = -numerator;return *this;}
    public:
        template<typename T2>
        bool operator==(const T2& value) const {return numerator == denominator*value;}
        bool operator==(const rational<T> &ra) const
		    {return denominator == ra.denominator && numerator == ra.numerator;}
        bool operator==(int value) const
    		    {return (denominator == 1) ? numerator == value : false;}
        bool operator==(long value) const
    		    {return (denominator == 1) ? numerator == value : false;}
        template<typename T2>
        bool operator>(const T2& value) const {return numerator > denominator*value;}
        template<typename T2>
        bool operator<(const T2& value) const {return numerator < denominator*value;}

        bool operator>(const rational<T>&) const;
        bool operator<(const rational<T>&) const;

    public:
	friend ostream &operator<<(ostream &out,const rational &ra)
        {
    	if(ra.denominator != (int)1)
                out << ra.numerator.to_double()/ra.denominator.to_double();
        //    out << ra.numerator << "/" << ra.denominator;
        else
            out << ra.numerator;
        return out;
        }
                };
#endif
//---------------------------------------------------------------------------
#include "rational.cpp"
//---------------------------------------------------------------------------


