/*
 * (c) Copyright 2004 Frank Yeah (pwipwi@ms66.hinet.net)
 *
 * To anyone who acknowledges that this file is provided "AS IS"
 * without any express or implied warranty:
 *     permission to use, copy, modify, and distribute this file
 * for any purpose is hereby granted without fee, provided that
 * the above copyright notice and this notice appears in all
 * copies. Frank Yeah makes no representations about the suitability
 * of this software for any purpose.
 */
//---------------------------------------------------------------------------
#ifndef PolynomialCPP
#define PolynomialCPP
//---------------------------------------------------------------------------
#include "polynomial.h"
//---------------------------------------------------------------------------
template <class T1,class T2>
element<T1,T2>::element(const string &sa)
{
const char *ca,*cb;
if(sa[0] == '-')
    ca = &sa[1];
else
    ca = &sa[0];

cb = ca;
    while(isdigit(*cb) || *cb == '.')
        cb++;
if(cb == ca)
    coefficient = (T1)1;
else
    {
    string sb(ca,cb);
    //coefficient = (T1)sb.c_str();
    coefficient = sb;
    }
if(sa[0] == '-')
    coefficient = -coefficient;
if(*cb == 0)
    {
    sign = 'x';
    exponent = (T2)0;
    return;
    }
if(*cb == '*')
    cb++;
sign = *cb;
cb++;
if(*cb != '^')
    {
    exponent = (T2)1;
    return;
    }
string sc(++cb,sa.end());
exponent = (T2)atol(sc.c_str());
}
//---------------------------------------------------------------------------
template <class T1,class T2>
element<T1,T2>& element<T1,T2>::operator=(const element<T1,T2>& value)
{
if(this != &value)
        {
        sign = value.sign;
        coefficient = value.coefficient;
        exponent = value.exponent;
        }
return *this;
}
//---------------------------------------------------------------------------
template <class T1,class T2>
ostream& operator<<(ostream &out,const element<T1,T2> &value)
{
if(value.Zero())
    return (out << '0');
if(value.exponent == 0)
    {
    out << value.coefficient;
    return out;
    }
else
    {
    if(value.coefficient != 1)
        out << value.coefficient;
    else
        {
        if(!(value.coefficient > 0))
            out <<'-';
        }
    out << value.sign;
    }
if(value.exponent != 1)
    {
    out << '^';
    if(value.exponent<0)
        out << '\(' << value.exponent << ')';
    else
        out << value.exponent;
    }
return out;
}
//---------------------------------------------------------------------------

template <class T1,class T2>
polynomial<T1,T2>::polynomial(const string &sa)
{
const char *ca,*cb;
ca = &sa[0];
cb = &sa[1];
while(1)
    {
    while(*cb != '+' && *cb != '-' && *cb != 0)
        cb++;
    string sb(ca,cb);
    element<T1,T2> ea(sb);
    *this += ea;
    if(*cb == 0)
        break;
    if(*cb != '-')
        cb++;
    ca = cb;
    cb++;
    }
}

//---------------------------------------------------------------------------
template <class T1,class T2>
void polynomial<T1,T2>::operator+=(element<T1,T2> const &value)
{
map<T2,element<T1,T2>,less<T2> >::iterator mia;
if((mia = m_terms.find(value.exponent)) != m_terms.end())
    (*mia).second.coefficient += value.coefficient;
else
    m_terms[value.exponent] = value;
}
//---------------------------------------------------------------------------

template <class T1,class T2>
void polynomial<T1,T2>::operator-=(element<T1,T2> const &value)
{
map<T2,element<T1,T2>,less<T2> >::iterator mia;
if((mia = m_terms.find(value.exponent)) != m_terms.end())
    (*mia).second.coefficient -= value.coefficient;
else
    {
    m_terms[value.exponent] = value;
    m_terms[value.exponent] = -m_terms[value.exponent];
    }
}
//---------------------------------------------------------------------------
template <class T1,class T2>
void polynomial<T1,T2>::operator*=(element<T1,T2> const &value)
{
map<T2,element<T1,T2>,less<T2> >::iterator mia = m_terms.begin();
while(mia != m_terms.end())
    {
    (*mia).second *= value;
    mia++;
    }
}
//---------------------------------------------------------------------------

template <class T1,class T2>
void polynomial<T1,T2>::operator/=(element<T1,T2> const &value)
{
map<T2,element<T1,T2>,less<T2> > :: iterator mia = m_terms.begin();
while(mia != m_terms.end())
    {
    (*mia).second /= value;
    mia++;
    }
}
//---------------------------------------------------------------------------
template <class T1,class T2>
void polynomial<T1,T2>::Check(void)
{
map<T2,element<T1,T2>,less<T2> > :: iterator mia = m_terms.begin();
if(m_terms.size() == 0)
    return;
do
    {
    if((*mia).second.Zero())
        m_terms.erase((--mia));
    else
        mia--;
    }
while(mia != m_terms.begin());
}
//---------------------------------------------------------------------------

template <class T1,class T2>
void polynomial<T1,T2>::operator+=(polynomial<T1,T2> const &value)
{
map<T2,element<T1,T2>,less<T2> > :: const_iterator mia = value.m_terms.begin();
while(mia != value.m_terms.end())
    {
    *this += (*mia).second;
    mia++;
    }
}
//---------------------------------------------------------------------------
template <class T1,class T2>
void polynomial<T1,T2>::operator-=(polynomial<T1,T2> const &value)
{
map<T2,element<T1,T2>,less<T2> > :: const_iterator mia = value.m_terms.begin();
while(mia != value.m_terms.end())
    {
    *this -= (*mia).second;
    mia++;
    }
}
//---------------------------------------------------------------------------
template <class T1,class T2>
bool polynomial<T1,T2>::operator==(polynomial<T1,T2> const &value)
{
map<T2,element<T1,T2>,less<T2> > :: const_iterator mia = value.m_terms.begin();
map<T2,element<T1,T2>,less<T2> > :: const_iterator mib = this->m_terms.begin();

if(this->m_terms.size() != value.m_terms.size())
        return false;
while(mia != value.m_terms.end())
    {
    if((*mia).second != (*mib).second)
        return false;
    mia++;
    mib++;
    }
return true;
}
//---------------------------------------------------------------------------
template <class T1,class T2>
void polynomial<T1,T2>::operator*=(polynomial<T1,T2> const &value)
{
polynomial<T1,T2> pa;
map<T2,element<T1,T2>,less<T2> > :: const_iterator mia = value.m_terms.begin();
while(mia != value.m_terms.end())
    {
    polynomial<T1,T2> pb = *this;
    pb *= (*mia).second;
    pa += pb;
    mia++;
    }
*this = pa;
}
//---------------------------------------------------------------------------
template <class T1,class T2>
void polynomial<T1,T2>::operator/=(polynomial<T1,T2> const &value)
{
if(value.m_terms.size() != 1)
    {
    throw "have not been implemented!!";
    }
*this /= (*(value.m_terms.begin())).second;
}
//---------------------------------------------------------------------------
template <class T1,class T2>
void polynomial<T1,T2>::operator^=(polynomial<T1,T2> const &value)
{
formula::exponent(*this,(*(value.m_terms.begin())).second.coefficient);
}
//---------------------------------------------------------------------------
template <class T1,class T2>
ostream& operator<<(ostream &out,
                const polynomial<T1,T2> &value)
{
map<T2,element<T1,T2>,less<T2> > ::const_iterator mia = value.m_terms.begin();
map<T2,element<T1,T2>,less<T2> > ::const_iterator mib = value.m_terms.end();
if(mia == mib)
    {
    out << '0';
    return out;
    }
mib--;
while(mia != mib)
    {
    out << (*mia).second;
        if(((*(++mia)).second).Positive())
            out << '+';
    }
out << (*mia).second;
return out;
}
//---------------------------------------------------------------------------


#endif
