/*
 * (c) Copyright 2004 Frank Yeah (pwipwi@ms66.hinet.net)
 *
 * To anyone who acknowledges that this file is provided "AS IS"
 * without any express or implied warranty:
 *     permission to use, copy, modify, and distribute this file
 * for any purpose is hereby granted without fee, provided that
 * the above copyright notice and this notice appears in all
 * copies. Frank Yeah makes no representations about the suitability
 * of this software for any purpose.
 */

//---------------------------------------------------------------------------

#ifndef PolynomialH
#define PolynomialH
//---------------------------------------------------------------------------
#include <map>
#include <iostream>

//---------------------------------------------------------------------------
using namespace std;

template <class T1,class T2>
class polynomial;
template <class T1,class T2>
class element;
template <class T1,class T2>
ostream& operator<<(ostream &out,const polynomial<T1,T2> &polynomial);
template <class T1,class T2>
ostream& operator<<(ostream &out,const element<T1,T2> &element);

template <class T1,class T2>
class element{
        friend class polynomial<T1,T2>;
        friend ostream& operator<< <>(ostream &out,const polynomial<T1,T2> &polynomial);
        friend ostream& operator<< <>(ostream &out,const element<T1,T2> &element);
    public:
        unsigned char sign;
        T1 coefficient;
        T2 exponent;
    public:
        element(void):sign('x'),coefficient(0),exponent(0){;}
        element(unsigned char isign,const T1 &ico,const T2 &iexp)
        :sign(isign),coefficient(ico),exponent(iexp){;}
        element(const string &sa);
        element(const element<T1,T2>& value){*this = value;}
    public:
        element<T1,T2>& operator=(const element<T1,T2>&);
    public:
        void operator*=(element<T1,T2> const &element)
            {this->coefficient*=element.coefficient;this->exponent += element.exponent;}
        void operator/=(element<T1,T2> const &element)
            {this->coefficient/=element.coefficient;this->exponent -= element.exponent;}
        template<typename T2>
        void operator ^=(const T2& value)
            {this->coefficient ^= value;this->exponent *= value;}
    public:
        template<typename T2>
        void operator*=(const T2& value){this->coefficient*=value;}
        template<typename T2>
        void operator/=(const T2& value){this->coefficient/=value;}
    public:
        element<T1,T2>& operator-(void){coefficient = -coefficient;return *this;}
        bool Zero(void) const{return coefficient == 0;}
        bool Positive(void) const{return coefficient>0;}
                };

template <class T1,class T2>
class polynomial{
        private:
            map<T2,element<T1,T2>,less<T2> > m_terms;
        public:
            polynomial(void){;}
            polynomial(int ia){element<T1,T2> ea('x',T1(ia),T2(0));m_terms[T2(0)] = ea;}
            polynomial(const string &sa);
            void operator+=(element<T1,T2> const &element);
            void operator-=(element<T1,T2> const &element);
            void operator*=(element<T1,T2> const &element);
            void operator/=(element<T1,T2> const &element);
            void operator+=(polynomial<T1,T2> const &polynomial);
            void operator-=(polynomial<T1,T2> const &polynomial);
            void operator*=(polynomial<T1,T2> const &polynomial);
            void operator/=(polynomial<T1,T2> const &polynomial);
            void operator^=(polynomial<T1,T2> const &polynomial);
            bool operator==(polynomial<T1,T2> const &polynomial);
        public:
            void Check();
            element<T1,T2> &ToConstant(){return m_terms[0];}
            friend ostream& operator<< <>(ostream &out,const polynomial<T1,T2> &polynomial);
                    };

//---------------------------------------------------------------------------
#endif
//---------------------------------------------------------------------------
#include "polynomial.cpp"
//---------------------------------------------------------------------------

